package	jp.co.sds_corp.ftp.service;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.ftp.FtpClient;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.log.CmnLogger;

public class FtpService extends AbstractService implements Runnable
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	transient FtpClient	ftp;
	String		host;
	String		dir;
	String		remote;
	String		action;
	String		passive;
	String		debug;
	Vector<String>	vecFile = new Vector<String>();
	int			timeout;
	transient Thread		thr;

	public void init( IConfiguration config ) throws Exception {
		ftp = new FtpClient();
		host = config.get( "host" );
		ftp.setHost( host );
		ftp.setUser( config.get( "user" ));
		ftp.setPassword( config.get( "password" ));
		timeout = config.getInteger( "timeout", 30 );
		ftp.setTimeout( timeout );
		passive = config.get( "passive", "false" );
		if ( passive.equals( "true" )) {
			ftp.setPassive( true );
		}
		debug = config.get( "debug", "false" );
		if ( debug.equals( "true" )) {
			ftp.setDebug( true );
		}

		dir = config.get( "dir", null );
		remote = config.get( "remotedir", null );
		action = config.get( "action" );
		Iterator<IConfiguration> itr = config.getChilds( "file" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			vecFile.add( c.get() );
		}
	}

	public void start() {
		setStatus( Status.RUN );
		thr = new Thread( this );
		thr.setName( getName() );
		thr.start();
		try {
			thr.join( timeout * 1000 );
		} catch ( InterruptedException ex ) {
		}
		if ( thr.isAlive()) {
			thr.interrupt();
			Thread.yield();
			logger.error( this, "ftp interrrupt" );
		}
	}

	public void run() {
		try {
			ftp.connect();
			ftp.type( FtpClient.TransferMode.BINARY );
			if ( remote != null ) {
				ftp.cd( remote );
			}
			for ( int i = 0; i < vecFile.size(); i++ ) {
				String file = (String )vecFile.get( i );
				String local;
				if ( dir != null ) {
					local = dir + "/" + file;
				} else {
					local = file;
				}
				if ( action.equals( "put" )) {
					ftp.put( file + ".tmp", local );
					ftp.rename( file + ".tmp", file );
					logger.debug( this, "file:" + local + "->" + file + " done." );
				} else if ( action.equals( "get" )) {
					ftp.get( file, local  + ".tmp" );
					new File( local+".tmp" ).renameTo( new File( local ));
					logger.debug( this, "file:" + local + "<-" + file + " done." );
				}
			}
			setStatus( Status.WAIT );
		} catch ( Throwable e ) {
			logger.error( this, "ftp error ", e );
			setStatus( Status.ERROR );
		} finally {
			try {
				ftp.disconnect();
			} catch ( Exception e ) {
			}
		}
	}

	public void stop() {
	}

	/**
	* タイムアウト時間(秒)(timeout)を設定します。
	* @param	var		設定するデータ
	*/
	public void setTimeout( int var ) {
		this.timeout = var;
	}
	/**
	* タイムアウト時間(秒)(timeout)を取得します。
	* @return	timeoutの内容
	*/
	public int getTimeout() {
		return timeout;
	}

	public String toString() {
		String ret = super.toString();
		ret += " host=[" + host + "] dir=[" + dir + "] remote=[" + remote + "] ";
		for ( int i = 0; i < vecFile.size(); i++ ) {
			ret += " file=[" + vecFile.get( i ) + "] ";
		}
		ret += " timeout=" + timeout;
		if ( passive != null && passive.equals( "true" )) {
			ret += " passive=" + passive;
		}
		if ( debug != null && debug.equals( "true" )) {
			ret += " debug=" + debug;
		}
		return ret;
	}
}
